class ImageSelector{
  //Class for image selection
  //Note: Save functions appear to freeze the program if the user attempts to save as a gif or jpg
  String extension;
  String dir;
  String tempDir;
  String result;

  //TAB ("\t") will be replaced with the name of the file
  String openPrompt;
  String openSuccess;
  String openExtensionFailMsg;
  String openNoSelectionMsg;
  String openUnknownFailMsg;
  
  String revertSuccess;
  String revertNoDirMsg;

  String savePrompt;
  String saveSuccess;
  String saveOverwriteMsg;
  String saveExtensionFailMsg;
  String saveExtensionAddMsg;
  String saveNoSelectionMsg;
  String saveUnknownFailMsg;
  
  String statusMessage;

  PImage loadedImage = null;
  PImage tempImage = null;

  ImageSelector(String fileExtension){
    extension = fileExtension;

    openPrompt = "Open a \"" + extension + "\" file. . .";
    openSuccess = "\"\t\" was successfully opened.";
    openExtensionFailMsg = "Selected file must be a \"" + extension + "\" file.";
    openNoSelectionMsg = "Request Canceled";
    openUnknownFailMsg = "An unknown error has occured while attempting to load \"\t\"";
    
    revertSuccess = "Reverted to last saved version of \"\t\"";
    revertNoDirMsg = "Failed to load image.";

    savePrompt = "Save image as. . .";
    saveSuccess = "\"\t\" was successfully saved.";
    saveExtensionFailMsg = "Save failed: Image must have \"" + extension + "\" extension.";
    saveExtensionAddMsg = "\"\t\" was successfully saved.  Extension: \"" + fileExtension + "\" was automaticaly added.";
    saveNoSelectionMsg = "Request Canceled";
    saveUnknownFailMsg = "An unknown error has occured while attempting to save \"\t\"";
  }
  
  //Save to dir
  void saveDir(){
    if(dir != null){
      loadedImage.save(dir);
      statusMessage = (replaceFileNameEscape(saveSuccess));
    }
    else{
      selectSave();
    }
  }

  //Ask user to select a new directory to save to
  //It will attempt to force the user to add the same extension as the one used
  void selectSave(){
    //Select output
    tempDir = selectOutput(savePrompt);
    //Check for selection
    if (tempDir != null){
      //Check for extension
      if (tempDir.substring(tempDir.length()-4).equals(extension)){
        loadedImage.save(dir);
        statusMessage = (replaceFileNameEscapeTemp(saveSuccess));
      }
      else{
        statusMessage = (replaceFileNameEscapeTemp(saveExtensionFailMsg));
      }
    }
    else{
      statusMessage = (replaceFileNameEscapeTemp(saveNoSelectionMsg));
    }
  }

  //Ask user to select a new directory to save to and add extension if not provided
  //This does not attempt to prevent the user from adding their own extension
  void selectSaveAutoReplaceExt(){
    //Select output
    tempDir = selectOutput(savePrompt);
    //Check for selection
    if (tempDir != null){
      //Check for extension
      if (tempDir.substring(tempDir.length()-4).equals(extension)){
        dir = tempDir;
        loadedImage.save(dir);
        statusMessage = (replaceFileNameEscapeTemp(saveSuccess));
      }
      else{
        tempDir = tempDir + extension;
        dir = tempDir;
        loadedImage.save(dir);
        statusMessage = (replaceFileNameEscapeTemp(saveExtensionAddMsg));
      }
    }
    else{
      statusMessage = (replaceFileNameEscapeTemp(saveNoSelectionMsg));
    }
  }

  //Revert (reset loaded image to file)
  void revert(){
    loadedImage = loadImage(dir);
    statusMessage = (replaceFileNameEscapeTemp(revertSuccess));
  }

  //Ask user to select a new image to open
  void selectOpen(){
    //Select input
    tempDir = selectInput(openPrompt);
    //Check for selection
    if (tempDir != null){
      //Check extension
      if (tempDir.substring(tempDir.length()-4).equals(extension)){
        tempImage = loadImage(tempDir, extension.substring(1));
        //Check for errors while loading
        if (tempImage.width != -1 && tempImage.height != -1 && tempImage != null){
          loadedImage = tempImage;
          dir = tempDir;
          statusMessage = (replaceFileNameEscapeTemp(openSuccess));
        }
        else{
          statusMessage = (replaceFileNameEscapeTemp(openUnknownFailMsg));
        }
      }
      else{
        statusMessage = (replaceFileNameEscapeTemp(openExtensionFailMsg));
      }
    }
    else{
      statusMessage = (replaceFileNameEscapeTemp(openNoSelectionMsg));
    }
  }

  String replaceFileNameEscape(String target){
    int lastIndexOfSearch = 0;
    while (target.indexOf ('\t', lastIndexOfSearch+1) != -1){
      if (target.indexOf('\t', lastIndexOfSearch+1) > lastIndexOfSearch){
        target = (target.substring(0, target.indexOf("\t")) + fileName(dir) + target.substring(target.indexOf("\t")+1));
      }
    }
    return(target);
  }

  String replaceFileNameEscapeTemp(String target){
    int lastIndexOfSearch = 0;
    while (target.indexOf ('\t', lastIndexOfSearch+1) != -1){
      if (target.indexOf('\t', lastIndexOfSearch+1) > lastIndexOfSearch){
        target = (target.substring(0, target.indexOf("\t")) + fileName(tempDir) + target.substring(target.indexOf("\t")+1));
      }
    }
    return(target);
  }

  String fileName(String target){
    int lastIndexOfSearch = 0;
    while (target.indexOf ('/', lastIndexOfSearch+1) != -1){
      if (target.indexOf('/', lastIndexOfSearch+1) > lastIndexOfSearch){
        lastIndexOfSearch = target.indexOf('/', lastIndexOfSearch+1);
      }
    }
    return target.substring(lastIndexOfSearch+1);
  }
}

