class MinecraftSkin extends ImageSelector {
  //Extended class for minecraft skins
  //Adds check for size
  String openSizeFailMsg;
  TexturedPlanes PlayerSkin = new TexturedPlanes();
  color transparent = color(255,255,255,0);
  
  MinecraftSkin() {
    super(".png");
    loadedImage = createImage(64, 32, ARGB);
    openSizeFailMsg = "Dimensions of \"\t\" are not 64 x 32";
    selectOpenSkin();
    PlayerSkin.source = loadedImage;
  }

  //Ask user to select a new image to open
  void selectOpenSkin() {
    //Select input
    tempDir = selectInput(openPrompt);
    //Check for selection
    if (tempDir != null) {
      //Check extension
      if (tempDir.substring(tempDir.length()-4).equals(extension)) {
        tempImage = loadImage(tempDir, extension.substring(1));
        //Check for errors while loading
        if (tempImage.width != -1 && tempImage.height != -1 && tempImage != null) {
          //Check image size
          if (tempImage.width == 64 && tempImage.height == 32) {
            loadedImage = tempImage;
            dir = tempDir;
            PlayerSkin.source = loadedImage;
            statusMessage = (replaceFileNameEscapeTemp(openSuccess));
          }
          else {
            statusMessage = (replaceFileNameEscapeTemp(openSizeFailMsg));
          }
        }
        else {
          statusMessage = (replaceFileNameEscapeTemp(openUnknownFailMsg));
        }
      }
      else {
        statusMessage = (replaceFileNameEscapeTemp(openExtensionFailMsg));
      }
    }
    else {
      statusMessage = (replaceFileNameEscapeTemp(openNoSelectionMsg));
    }
  }
  
  void revertSkin(){
    loadedImage = loadImage(dir);
    PlayerSkin.source = loadedImage;
  }
  
  void quickfix(){
    loadedImage.loadPixels();
    for(int i=0; i<2048; i++){
      if(alpha(loadedImage.pixels[i]) == 0){
        loadedImage.pixels[i] = transparent;
      }
    }
    loadedImage.updatePixels();
    PlayerSkin.source = loadedImage;
    statusMessage = "All transparent pixels made transparent";
  }
  
  void quickfix2(){
    loadedImage.loadPixels();
    for(int i=0; i<2048; i++){
      if(loadedImage.pixels[i] == color(0,0,0)){
        loadedImage.pixels[i] = transparent;
      }
    }
    loadedImage.updatePixels();
    PlayerSkin.source = loadedImage;
    statusMessage = "All black pixels made transparent";
  }
}

class TexturedPlanes {
  //Class for enlaging parts of an image to form planes in a 3D space
  PImage source;
  TexturedPlanes() {
    
  }

  //Draw Planes perpendicular to the X axis
  //First letter is which axis to use as X axis. Capital is positive, lowercase is negative.
  //Second letter is which axis to use as Y axis. Capital is positive, lowercase is negative.
  void YZPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(0, pixelHeight, pixelWidth);
        translate(0, pixelHeight, 0);
      }
      translate(0, -pixelHeight*(skinX2-skinX1), pixelWidth);
    }
    popMatrix();
  }

  void YzPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(0, pixelHeight, pixelWidth);
        translate(0, pixelHeight, 0);
      }
      translate(0, -pixelHeight*(skinX2-skinX1), -pixelWidth);
    }
    popMatrix();
  }

  void yZPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(0, pixelHeight, pixelWidth);
        translate(0, -pixelWidth, 0);
      }
      translate(0, pixelHeight*(skinX2-skinX1), pixelWidth);
    }
    popMatrix();
  }

  void yzPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(0, pixelHeight, pixelWidth);
        translate(0, -pixelWidth, 0);
      }
      translate(0, pixelHeight*(skinX2-skinX1), -pixelWidth);
    }
    popMatrix();
  }

  //Flip Across Diagonal

  void ZYPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(0, pixelHeight, pixelWidth);
        translate(0, 0, pixelWidth);
      }
      translate(0, pixelHeight, -pixelWidth*(skinX2-skinX1));
    }
    popMatrix();
  }

  void zYPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(0, pixelHeight, pixelWidth);
        translate(0, 0, -pixelWidth);
      }
      translate(0, pixelHeight, pixelWidth*(skinX2-skinX1));
    }
    popMatrix();
  }

  void ZyPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(0, pixelHeight, pixelWidth);
        translate(0, 0, -pixelWidth);
      }
      translate(0, -pixelHeight, pixelWidth*(skinX2-skinX1));
    }
    popMatrix();
  }

  void zyPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(0, pixelHeight, pixelWidth);
        translate(0, 0, -pixelWidth);
      }
      translate(0, -pixelHeight, pixelWidth*(skinX2-skinX1));
    }
    popMatrix();
  }

  //Draws Planes perpendicular to the Y axis
  //First letter is which axis to use as X axis. Capital is positive, lowercase is negative.
  //Second letter is which axis to use as Y axis. Capital is positive, lowercase is negative.
  void XZPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, 0, pixelHeight);
        translate(pixelWidth, 0, 0);
      }
      translate(-pixelWidth*(skinX2-skinX1), 0, pixelHeight);
    }
    popMatrix();
  }

  void xZPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, 0, pixelHeight);
        translate(-pixelWidth, 0, 0);
      }
      translate(pixelWidth*(skinX2-skinX1), 0, pixelHeight);
    }
    popMatrix();
  }

  void XzPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, 0, pixelHeight);
        translate(pixelWidth, 0, 0);
      }
      translate(-pixelWidth*(skinX2-skinX1), 0, -pixelHeight);
    }
    popMatrix();
  }

  void xzPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, 0, pixelHeight);
        translate(-pixelWidth, 0, 0);
      }
      translate(pixelWidth*(skinX2-skinX1), 0, -pixelHeight);
    }
    popMatrix();
  }

  //Flip Across Diagonal

  void ZXPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, 0, pixelHeight);
        translate(0, 0, pixelHeight);
      }
      translate(pixelWidth, 0, -pixelHeight*(skinX2-skinX1));
    }
    popMatrix();
  }

  void zXPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, 0, pixelHeight);
        translate(0, 0, -pixelHeight);
      }
      translate(pixelWidth, 0, pixelHeight*(skinX2-skinX1));
    }
    popMatrix();
  }

  void ZxPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, 0, pixelHeight);
        translate(0, 0, pixelHeight);
      }
      translate(-pixelWidth, 0, -pixelHeight*(skinX2-skinX1));
    }
    popMatrix();
  }

  void zxPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, 0, pixelHeight);
        translate(0, 0, -pixelHeight);
      }
      translate(-pixelWidth, 0, pixelHeight*(skinX2-skinX1));
    }
    popMatrix();
  }

  //Draws Planes perpendicular to the Z axis
  //First letter is which axis to use as X axis. Capital is positive, lowercase is negative.
  //Second letter is which axis to use as Y axis. Capital is positive, lowercase is negative.
  void XYPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, pixelHeight, 0);
        translate(pixelWidth, 0, 0);
      }
      translate(-pixelWidth*(skinX2-skinX1), pixelHeight, 0);
    }
    popMatrix();
  }

  void XyPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, pixelHeight, 0);
        translate(pixelWidth, 0, 0);
      }
      translate(-pixelWidth*(skinX2-skinX1), -pixelHeight, 0);
    }
    popMatrix();
  }

  void xYPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, pixelHeight, 0);
        translate(-pixelWidth, 0, 0);
      }
      translate(pixelWidth*(skinX2-skinX1), pixelHeight, 0);
    }
    popMatrix();
  }

  void xyPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, pixelHeight, 0);
        translate(-pixelWidth, 0, 0);
      }
      translate(pixelWidth*(skinX2-skinX1), -pixelHeight, 0);
    }
    popMatrix();
  }

  //Flip Across Diagonal

  void YXPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, pixelHeight, 0);
        translate(0, pixelHeight, 0);
      }
      translate(pixelWidth, -pixelHeight*(skinX2-skinX1), 0);
    }
    popMatrix();
  }

  void yXPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, pixelHeight, 0);
        translate(0, pixelHeight, 0);
      }
      translate(-pixelWidth, -pixelHeight*(skinX2-skinX1), 0);
    }
    popMatrix();
  }

  void YxPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, pixelHeight, 0);
        translate(0, -pixelHeight, 0);
      }
      translate(pixelWidth, pixelHeight*(skinX2-skinX1), 0);
    }
    popMatrix();  
  }

  void yxPlane(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    pushMatrix();
    translate(x, y, z);
    for (int bfy=skinY1; bfy<skinY2; bfy++) {
      for (int bfx=skinX1; bfx<skinX2; bfx++) {
        fill(source.pixels[bfy*64+bfx]);
        box(pixelWidth, pixelHeight, 0);
        translate(0, -pixelHeight, 0);
      }
      translate(-pixelWidth, pixelHeight*(skinX2-skinX1), 0);
    }
    popMatrix();
  }
  
  int mousePixelX;
  int mousePixelY;
  
  void xzPlaneMouse(int x, int y, int z, int skinX1, int skinY1, int skinX2, int skinY2, int pixelWidth, int pixelHeight) {
    mousePixelX = -1;
    mousePixelY = -1;
    pushMatrix();
    translate(x, y, z);
    //Plane
      for(int bfy = skinY1; bfy < skinY2; bfy++) {
        for(int bfx = skinX1; bfx < skinX2; bfx++) {
          //Pixel
            fill(source.pixels[bfy*64+bfx]);
            for(int xw = 0; xw < pixelHeight; xw++){
              for(int yh = 0; yh < pixelWidth; yh++){
                box(1, 0, 1);
                translate(1, 0, 0);
                if(mouseX == round(screenX(0, 0, 0)) && mouseY == round(screenY(0, 0, 0))){
                  println(bfy + "," + bfx);
                  mousePixelX = bfx;
                  mousePixelY = bfy;
                }
              }
              translate(-pixelWidth, 0, -1);
            }
          translate(-pixelWidth, 0, pixelHeight);
        }
        translate(pixelWidth*(skinX2-skinX1), 0, -pixelHeight);
      }
    popMatrix();
  }
}
